﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Primeval
{
    enum State
    {
        BREED,
        SPAWN,
        BATTLE,
        WAIT
    }

    /// <summary>
    /// This class keeps track of what state the game is in
    /// </summary>
    static class GameState
    {
        private static State mcurstate;
        public static State currentState
        {
            get { return mcurstate; }
            set
            {
                mcurstate = value;
                newState = true;
                currentStateLength = TimeSpan.Zero;
            }
        }

        public static TimeSpan currentStateLength;

        public static bool newState;

        public static State CurrentState { get { return currentState; } }

        static GameState()
        {
            currentState = State.WAIT;
        }

        public static void Reset()
        {
            currentState = State.WAIT;
        }

        public static void Update(GameTime gameTime)
        {
            currentStateLength += gameTime.ElapsedGameTime;

            switch (currentState)
            {
                case State.BREED:
                    if (newState)
                    {
                        GenePool.NextGeneration();
                        Game1.player.NewRound();

                        MessageDisplay.Clear();
                        MessageDisplay.PostMessage("Thank you, subject.");
                        MessageDisplay.PostMessage("Subject shields recharged.");
                        MessageDisplay.PostMessage("");
                        MessageDisplay.PostMessage("Breeding Generation "
                            + MessageDisplay.PadNum(GenePool.generationCount, 4));
                        MessageDisplay.PostMessage("  Selected products:");
                        MessageDisplay.PostMessage("   [Right Panel]");

                        newState = false;
                    }
                    if (currentStateLength.TotalSeconds > 5 && MessageDisplay.Waiting)
                    {
                        currentState = State.SPAWN;
                        MessageDisplay.PostMessage("Breeding threads complete.");
                    }
                    break;
                case State.SPAWN:
                    if (newState)
                    {
                        GenePool.SpawnBot();
                        MessageDisplay.Clear();
                        MessageDisplay.PostMessage("Generation "
                            + MessageDisplay.PadNum(GenePool.generationCount, 4));
                        MessageDisplay.PostMessage(
                            "Dispensing product " + GenePool.GenerationSize + "/" + GenePool.generationSize + ".");
                        newState = false;
                    }
                    if (currentStateLength.TotalSeconds > 4 && MessageDisplay.Waiting)
                    {
                        currentState = State.BATTLE;
                    }
                    break;
                case State.BATTLE:
                    if (newState)
                    {

                        newState = false;
                    }
                    if (Bot.BotCount == 0 && !Game1.player.Die)
                    {
                        currentState = State.WAIT;
                        MessageDisplay.PostMessage("Product destroyed.");
                    }
                    break;
                case State.WAIT:
                    if (newState)
                    {
                        newState = false;
                    }
                    if (currentStateLength.TotalSeconds > 2 && MessageDisplay.Waiting)
                    {
                        if (GenePool.GenerationOver)
                            currentState = State.BREED;
                        else
                            currentState = State.SPAWN;
                    }
                    break;
            }
        }

        public static void Draw(SpriteBatch spriteBatch)
        {
            switch (currentState)
            {
                case State.BREED:
                    break;
                case State.SPAWN:
                    break;
                case State.BATTLE:
                    break;
                case State.WAIT:
                    break;
            }
        }
    }
}
