﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Primeval
{
    /// <summary>
    /// This class represents the player on the screen
    /// </summary>
    public class Player
    {
        public Vector2 position;

        private Vector2 velocity;

        public const int radius = 25;

        private const float acceleration = 2000;

        private const float decceleration = 2000;

        private const float maxspeed = 450;

        private TimeSpan fireCooldown;

        private static TimeSpan fireCooldownTotal = TimeSpan.FromSeconds(0.2f);

        private float health;

        private float shield;

        private static Texture2D image;

        public bool Die { get { return health <= 0; } }

        public Player()
        {
            health = 130;
            shield = 130;

            position = Game1.playerSpawnPoint;
        }

        public void Reset()
        {
            shield = 130;
            health = 130;

            position = Game1.playerSpawnPoint;
        }

        public void NewRound()
        {
            shield = 130;
        }

        public void Damage(float damage)
        {
            if (Die) return;

            if (shield >= damage)
            {
                shield -= damage;
            }
            else
            {
                health -= damage - shield;
                shield = 0;
            }

            if (Die)
            {
                //Sound
                Game1.soundbank.PlayCue("BotDie");

                //Display
                MessageDisplay.Clear();
                MessageDisplay.PostMessage("Exercise concluded.");
                MessageDisplay.PostMessage("Thank you for your time.");
                MessageDisplay.PostMessage("Reimbursement will be sent");
                MessageDisplay.PostMessage(" to the next-of-kin.");

                //Fragments
                for (int c = 0; c < radius / 10; c++)
                    new Particle(
                        ParticleType.FRAG1 + Game1.random.Next(3),
                        position,
                        new Vector2(Game1.random.Next(400) - 200, -Game1.random.Next(300) - 200),
                        new Vector2(0, 250),
                        TimeSpan.FromSeconds(1.5),
                        (float)Game1.random.NextDouble() * 6,
                        Color.White,
                        ParticleType.SMOKE,
                        TimeSpan.FromMilliseconds(60),
                        radius * 2 / 100f);

                //Fire
                for (int c = 0; c < 8; c++)
                {
                    Vector2 velocity = new Vector2(
                        Game1.random.Next(100) - 50,
                        -Game1.random.Next(300));
                    new Particle(
                        ParticleType.FLAME, position,
                        velocity,
                        new Vector2(0, 0),
                        TimeSpan.FromMilliseconds(500),
                        false);
                    new Particle(
                        ParticleType.SMOKE, position,
                        velocity,
                        new Vector2(0, 0),
                        TimeSpan.FromMilliseconds(1000),
                        true);
                }
            }
        }

        public static void LoadContent()
        {
            image = Game1.content.Load<Texture2D>("botbase");
        }

        public void Update(GameTime gameTime)
        {
            if (Die) return;

            KeyboardState kstate = Keyboard.GetState();
            GamePadState gstate = GamePad.GetState(PlayerIndex.One);
            MouseState mstate = Mouse.GetState();

            bool moved = false;

            //Accelerate around
            if (gstate.IsConnected)
            {
                if (gstate.ThumbSticks.Left != Vector2.Zero)
                {
                    moved = true;
                    velocity += Vector2.Normalize(gstate.ThumbSticks.Left)
                        * acceleration * (float)gameTime.ElapsedGameTime.TotalSeconds;
                }
            }
            else
            {
                if (kstate.IsKeyDown(Keys.A) || kstate.IsKeyDown(Keys.Left))
                {
                    moved = true;
                    velocity.X -= acceleration * (float)gameTime.ElapsedGameTime.TotalSeconds;
                }
                if (kstate.IsKeyDown(Keys.D) || kstate.IsKeyDown(Keys.Right))
                {
                    moved = true;
                    velocity.X += acceleration * (float)gameTime.ElapsedGameTime.TotalSeconds;
                }

                if (kstate.IsKeyDown(Keys.W) || kstate.IsKeyDown(Keys.Up))
                {
                    moved = true;
                    velocity.Y -= acceleration * (float)gameTime.ElapsedGameTime.TotalSeconds;
                }
                if (kstate.IsKeyDown(Keys.S) || kstate.IsKeyDown(Keys.Down))
                {
                    moved = true;
                    velocity.Y += acceleration * (float)gameTime.ElapsedGameTime.TotalSeconds;
                }
            }

            //Deccelerate
            if (!moved && velocity != Vector2.Zero)
            {
                int sx = Math.Sign(velocity.X);
                int sy = Math.Sign(velocity.Y);
                velocity -= Vector2.Normalize(velocity) * decceleration
                    * (float)gameTime.ElapsedGameTime.TotalSeconds;
                if (sx != Math.Sign(velocity.X) || sy != Math.Sign(velocity.Y))
                    velocity = Vector2.Zero;
            }

            //Restrict speed
            if (velocity.Length() > maxspeed)
                velocity = Vector2.Normalize(velocity) * maxspeed;

            //Move
            position += velocity * (float)gameTime.ElapsedGameTime.TotalSeconds;

            //Out of bounds
            if (position.X < Game1.map.Left) position.X = Game1.map.Left;
            if (position.Y < Game1.map.Top) position.Y = Game1.map.Top;
            if (position.X > Game1.map.Right) position.X = Game1.map.Right;
            if (position.Y > Game1.map.Bottom) position.Y = Game1.map.Bottom;

            //Fire
            if ((gstate.IsConnected && (gstate.IsButtonDown(Buttons.LeftTrigger)
                || gstate.IsButtonDown(Buttons.RightTrigger))) ||
                mstate.LeftButton == ButtonState.Pressed)
            {
                fireCooldown -= gameTime.ElapsedGameTime;
                if (fireCooldown <= TimeSpan.Zero)
                {
                    Vector2 firedir;
                    if (gstate.IsConnected)
                        firedir = gstate.ThumbSticks.Right;
                    else
                        firedir = Game1.TransformToData(new Vector2(mstate.X, mstate.Y)) - position;

                    new Bullet(650, 3, firedir);

                    fireCooldown = fireCooldownTotal;
                }
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            if (Die) return;

            float dist = Math.Min(512, Vector2.Distance(position, Game1.fieldCenter));
            float r = 1 - (dist / 512f) * (dist / 512f);

            spriteBatch.Draw(
                image,
                Game1.TransformToDraw(position),
                null,
                new Color(r, r, r, 1),
                0,
                new Vector2(image.Width, image.Height) / 2,
                radius * 2 / 100f,
                SpriteEffects.None,
                1f);

            //Health UI
            spriteBatch.Draw(
                Game1.pixel,
                new Rectangle(
                    20, 700, (int)(health * 2), 15),
                Color.Red);

            spriteBatch.Draw(
                Game1.pixel,
                new Rectangle(
                    20, 680, (int)(shield * 2), 15),
                Color.Blue);
        }
    }
}
